(****************************************************************************)
(* Userdatenbank fr das Terminal von MCUT.                                 *)
(* Ausserdem Autorouter-Funtkionen                                          *)
(*                                                                          *)
(* Diese Unit ist Teil des MCUT-Sourcecodes, alle Rechte liegen bei         *)
(* DG4IAD und DG6VJ                                                         *)
(****************************************************************************)

UNIT TERMUSER ;

INTERFACE

USES IADVAR;

TYPE
     TTermUserRec=RECORD
         CALL:STRING[10];
         SSID:BYTE;
         FLAGS:LONGINT;
         Name:STRING[20];
         TYP:CHAR;
         LastChange:LONGINT;
         Level:BYTE; (* wegen kompatibilitt *)
         Einstieg:STRING[10];
         NETZWERK:STRING[10];
         FlexPassWord:LONGINT;
         BayPassWord:STRING[40];
         Directory:STRING[70];
         Remotes:LONGINT;
         remotes2:LONGINT;
         paclen:BYTE;
         reserve:ARRAY[1..1024] OF BYTE;
     END;

FUNCTION GetTermUser(VAR U:TTermUserRec):BOOLEAN;
FUNCTION SetTermUser(U:TTermUserRec):BOOLEAN;
FUNCTION LoadTermUser(CHAN:BYTE):BOOLEAN;
FUNCTION SaveTermUser(CHAN:BYTE):BOOLEAN;
PROCEDURE ClearRec(VAR U:TTermUserRec);
{$IFDEF FPK}   
FUNCTION InitRouter(c:BYTE;my,CALL,Einstieg,Netzwerk:STRING10):BOOLEAN;
{$ELSE}
FUNCTION InitRouter(c:BYTE;my,CALL,Einstieg,Netzwerk:STRING):BOOLEAN;
{$ENDIF}

PROCEDURE DoRouting(c:BYTE;S:STRING);
IMPLEMENTATION

USES
     {$IFDEF FLEXNET}
     FLXAPP,
     {$ENDIF}

     {$IFDEF WA8DED}
     TNC,
     {$ENDIF}
     MYSTRING,MCUTLIB2,TERMSCAN,TOOLBOX,PACKET;

VAR
  {$IFDEF FPK}  
   U_File:FILE;
   {$ELSE}
   U_File:FILE OF TTermUserRec;
  {$ENDIF}
  U_Rec:TTermUserRec;


CONST
   FileID:STRING[10]='TRMU-0003';

PROCEDURE ClearRec(VAR U:TTermUserRec);
BEGIN
  FILLCHAR(U,SizeOf(TTermUserRec),0);
  WITH U DO
  BEGIN
    level:=2;
    flags:=default_flags;
    remotes:=default_remotes;
    paclen:=236;
  END;
END;

FUNCTION OpenFile:BOOLEAN;
VAR S:STRING[10];
BEGIN
  OPENFILE:=TRUE;
   ASSIGN(U_File,qrvpath+'termuser.mct');
  {$I-}
  {$IFDEF FPK}
   RESET(U_FILE,SizeOf(TTermUserRec));
  {$ELSE}	 
  RESET(U_File);
  {$ENDIF} 
  {$I+}
  IF IORESULT<>0 THEN
  BEGIN
     {$IFDEF FPK}
     REWRITE(U_FILE,SizeOf(TTermUserRec));
     {$ELSE}	 
     REWRITE(U_File);
     {$ENDIF}
    IF IORESULT<>0 THEN
    BEGIN
      OpenFile:=FALSE;
      EXIT;
    END;
    Move(FileID,U_Rec,11);
    {$IFDEF FPK}
     BlockWrite(U_File,U_Rec,1);
    {$ELSE} 
     WRITE(U_File,U_Rec);
    {$ENDIF}
    SEEK(U_File,0);
  END;
  IF FileSize(U_File)=0 THEN
  BEGIN
    Move(FileID,U_Rec,11);
    {$IFDEF FPK}
     BlockWrite(U_File,U_Rec,1);
    {$ELSE} 
     WRITE(U_File,U_Rec);
    {$ENDIF}
    END ELSE
    BEGIN
    {$IFDEF FPK}
     BlockRead(U_File,U_Rec,1);
    {$ELSE} 
     Read(U_File,U_Rec);
    {$ENDIF}

    Move(U_Rec,s,11);
    IF S<>FileID THEN
    BEGIN
      CLOSE(U_File);
      OpenFile:=FALSE;
      Exit;
    END;
  END;
  ClearRec(U_Rec);
END;

FUNCTION GetTermUser(VAR U:TTermUserRec):BOOLEAN;
VAR
   Call_found : BOOLEAN;
   ssid,err   : integer;
   tmp_u      : TTermUserRec;
   l,r	      : string[10];
   numread    : longint;
BEGIN
  r:=gross(u.call);
  ssid:=u.ssid;
  ClearRec(u);
  u.call:=r;
  u.ssid:=ssid;
  IF find(u.Call,'-',l,r) THEN
  BEGIN
    Val(r,SSID,err);
    u.call:=l;
    u.ssid:=ssid;
  END;
  Call_Found:=FALSE;
  GetTermUser:=FALSE;
  IF NOT OpenFile THEN
    EXIT;
   numread:=1;
  WHILE (NOT EOF(U_File)) AND (numread>0) DO
  BEGIN
    {$IFDEF FPK}
     BlockRead(U_File,U_Rec,1,numread);
     if numread=0 then break;
    {$ELSE} 
     Read(U_File,U_Rec);
    {$ENDIF}
    IF (U.CALL=U_Rec.CALL) THEN
    BEGIN
      IF (U_rec.SSID=U.ssid) THEN
      BEGIN
        U:=U_Rec;
        GetTermUser:=TRUE;
        CLOSE(U_File);
        Exit;
      END ELSE
      IF (U_Rec.FLAGS AND TU_SAVESSID)=0 THEN
      BEGIN
        tmp_U:=U_Rec;
        Call_Found:=TRUE;
      END;
    END;
  END;
  IF Call_found THEN
  BEGIN
    tmp_u.ssid:=u.ssid;
    U:=tmp_u;
    U.FLAGS:=U.FLAGS AND NOT TU_SAVESSID;
    GetTermUser:=TRUE;
  END;
  CLOSE(U_File);
END;

FUNCTION SetTermUser(U:TTermUserRec):BOOLEAN;
VAR merkpos	      : WORD; l,r:string[10];
    ssid,err	      : integer;
   numread	      : longint;
BEGIN
  SetTermUser:=FALSE;
  IF find(u.Call,'-',l,r) THEN
  BEGIN
    Val(r,SSID,err);
    u.call:=l;
    u.ssid:=ssid;
  END;
  u.call:=gross(u.call);
  IF NOT OpenFile THEN
    EXIT;
  U.lastChange:=CurrIxTime;
  merkpos:=$FFFF;
   numread:=1;
  WHILE (NOT EOF(U_File)) AND (numread>0) DO
  BEGIN
    {$IFDEF FPK}
     BlockRead(U_File,U_Rec,numread);
     if numread=0 then break;
    {$ELSE} 
     Read(U_File,U_Rec);
    {$ENDIF}
    IF (U.CALL=U_Rec.Call) AND (U.SSID <> U_Rec.SSID) THEN
      merkpos:=PRED(filepos(u_file));
    IF (U.CALL=U_Rec.CALL) AND (U.SSID = U_Rec.SSID) THEN
    BEGIN
      SetTermUser:=TRUE;
      U.SSID:=U_REC.SSID;
      SEEK(U_File,PRED(FilePos(U_File)));
    {$IFDEF FPK}
     BlockWrite(U_File,U,1);
    {$ELSE} 
     WRITE(U_File,U);
    {$ENDIF}
      CLOSE(U_File);
      Exit;
    END;
  END;
  IF (merkpos<$FFFF) AND ((U.Flags AND TU_SAVESSID)=0) THEN
    SEEK(U_File,Merkpos);
    {$IFDEF FPK}
     BlockWrite(U_File,U,1);
    {$ELSE} 
     WRITE(U_File,U);
    {$ENDIF}

  CLOSE(U_File);
END;


FUNCTION LoadTermUser(CHAN:BYTE):BOOLEAN;
VAR U:TTermUserRec;SSID:BYTE;SSIDMASK:WORD;l,r:STRING[10];
    err:integer;
BEGIN
  LoadTermUser:=FALSE;
  IF KS[chan]=NIL THEN exit;
  ClearRec(u);
  IF find(KS[chan]^.Call,'-',l,r) THEN
  BEGIN
    Val(r,SSID,err);
    u.call:=l;
    u.ssid:=ssid;
  END ELSE
  BEGIN
    u.call:=KS[chan]^.call;
    u.ssid:=0;
  END;
  IF NOT GetTermUser(u) THEN
    KS[chan]^.FLAGS:=KS[chan]^.FLAGS OR KS_UNKNOWN;
  LoadTermUser:=TRUE;
  WITH KS[chan]^ DO
  BEGIN
    flags:=TU_2_KS(u.flags,flags);
    remotes:=u.remotes;
    lev:=u.level;
    Name:=u.name;
  END;
END;

FUNCTION SaveTermUser(CHAN:BYTE):BOOLEAN;
VAR U:TTermUserRec;SSID:BYTE;SSIDMASK:WORD;l,r:STRING[10];
    err:integer;
BEGIN
  SaveTermUser:=FALSE;
  IF KS[chan]=NIL THEN exit;
  ClearRec(u);
  IF find(KS[chan]^.Call,'-',l,r) THEN
  BEGIN
    Val(r,SSID,err);
    u.call:=l;
    u.ssid:=ssid;
  END ELSE
  BEGIN
    u.call:=KS[chan]^.call;
    u.ssid:=0;
  END;
  GetTermUser(u);
  WITH KS[chan]^ DO
  BEGIN
    U.FLAGS:=KS_2_TU(flags,u.flags);
    u.remotes:=remotes;
    u.level:=lev;
    U.Name:=name;
  END;
  SetTermUser(U);
END;



PROCEDURE LoadNextRoute(C:BYTE);
VAR B,ca,da:BYTE; S:STRING[80];
BEGIN
  WITH KS[c]^.ConnScriptData^ DO
  BEGIN
    FOR B:= 1 to 5 DO
    BEGIN
      ConnMessages[B]:='';
      DiscMessages[B]:='';
    END;
    ca:=0;
    da:=0;
    WHILE (Script<>NIL) DO
    BEGIN
      PopQueue(Script,b,s);
      CASE S[1] OF
        '+':IF ca<5 THEN
            BEGIN
              INC(ca);
              ConnMessages[ca]:=COPY(s,2,255);
            END;
        '-':IF da<5 THEN
            BEGIN
              INC(da);
              DiscMessages[ca]:=COPY(s,2,255);
            END;
        '!':BEGIN
              S:=COPY(s,2,255);
              B:=POS('%%ENTRY',GROSS(S));
              IF B>0 THEN
              BEGIN
                DELETE(S,B,7);
                INSERT(DIGI,S,B);
              END;
              B:=POS('%%USER',GROSS(S));
              IF B>0 THEN
              BEGIN
                DELETE(S,B,6);
                INSERT(USER,S,B);
              END;
              Conn_BEFEHL:=S;
              EXIT;
            END;
       END;
    END;
  END;
END;



{$IFDEF FPK}   
FUNCTION InitRouter(c:BYTE;my,CALL,Einstieg,Netzwerk:STRING10):BOOLEAN;
{$ELSE}
FUNCTION InitRouter(c:BYTE;my,CALL,Einstieg,Netzwerk:STRING):BOOLEAN;
{$ENDIF}

VAR U:TTermUserRec;F:TEXT;
    S:STRING;OK:BOOLEAN; initdest:STRING;
    B,ssid:BYTE;
    qsonum:WORD;

PROCEDURE mkc;
BEGIN
  WITH KS[c]^.ConnScriptData^ DO
  BEGIN
    B:=POS('%%ENTRY',GROSS(InitDest));
    IF B>0 THEN
    BEGIN
      DELETE(InitDest,B,7);
      INSERT(DIGI,InitDest,B);
    END;
    B:=POS('%%USER',GROSS(InitDest));
    IF B>0 THEN
    BEGIN
      DELETE(InitDest,B,6);
      INSERT(USER,InitDest,B);
    END;
  END;
  KS[c]^.FLAGS:=KS[c]^.FLAGS OR KS_OWNCON;

  {$IFDEF FLEXNET}

  IF device=1 THEN
  BEGIN
    IF KS[c]^.LinkState=0 THEN
    BEGIN
      ein_s:=my;
      qsonum:=l2_Pcheck_path(ein_s,initdest);
      WHILE (qsonum>0) AND (l2_state(qsonum)>1) DO
      BEGIN
        IF FIND(ein_s,'-',L,R) THEN
          val(R,ssid,err)
        ELSE
        BEGIN
          l:=ein_s;
          ssid:=0;
        END;
        INC(SSID);
        IF ssid>15 THEN
        BEGIN
          IF KS[C]^.ConnScriptData^.Script<>NIL THEN
            PurgeQueue( KS[C]^.ConnScriptData^.script);
          KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
          DISPOSE(KS[c]^.ConnScriptData);
          exit;
        END;
        str(ssid,r);
        ein_s:=l+'-'+r;
        qsonum:=l2_Pcheck_path(ein_S,initdest);
      END;
      KS[c]^.Stream:=l2_pconnect(ein_s,initdest);
   END;
   IF KS[c]^.Stream=0 THEN
   BEGIN
     IF KS[C]^.ConnScriptData^.Script<>NIL THEN
       PurgeQueue( KS[C]^.ConnScriptData^.script);
     KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
     DISPOSE(KS[c]^.ConnScriptData);
     Exit;
   END;
  END;

  {$ENDIF}
  {$IFDEF WA8DED}

  IF device=0 THEN
  BEGIN
    sendcmd(c,'I '+my);
    IF sendcmd(c,'C '+initdest)<>'' THEN
    BEGIN
      IF KS[C]^.ConnScriptData^.Script<>NIL THEN
        PurgeQueue( KS[C]^.ConnScriptData^.script);
      KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
      DISPOSE(KS[c]^.ConnScriptData);
      Exit;
    END;
  END;
  {$ENDIF}

  KS[c]^.ConnScriptData^.connmessages[1]:='!!!-**HANDLE$%';
   {InitRouter:=TRUE;
}
END;

BEGIN
  InitRouter:=FALSE;
  IF KS[c]=NIL THEN exit;
  OK:=FALSE;
  IF KS[c]^.ConnScriptData=NIL THEN
  BEGIN
    NEW(KS[c]^.ConnScriptData);
    Fillchar(KS[c]^.ConnScriptData^,SizeOf(TConnScriptData),0);
    KS[c]^.ConnScriptData^.StartTime:=CurrIxTime;
    KS[c]^.ConnScriptData^.LastStatus:=KS[c]^.Status;
    KS[c]^.Status:=25;
  END;
  U.CALL:=CALL;
  IF (einstieg='') THEN
    IF GetTermUser(U) THEN
    BEGIN
      Netzwerk:=U.Netzwerk;
      Einstieg:=U.Einstieg;
    END;
  netzwerk:=rtrim(ltrim(gross(netzwerk)));
  einstieg:=rtrim(ltrim(gross(einstieg)));
  IF NOT ((einstieg='') AND (netzwerk='')) THEN
  BEGIN
    ASSIGN(f,qrvpath+'NETWORKS.MCT');
    {$I-}
    RESET(f);
    IF IOResult<>0 THEN
    BEGIN
      KS[c]^.Status:=KS[c]^.ConnScriptData^.LastStatus;
      DISPOSE(KS[c]^.ConnScriptData);
      KS[c]^.ConnScriptData:=NIL;

      EXIT;
    END;
    WHILE NOT EOF(F) DO
    BEGIN
      ReadLn(f,s);
      s:=ltrim(s);
      IF NOT OK THEN
      BEGIN
         IF GROSS(COPY(s,1,7))='#DEFINE' THEN
         BEGIN
           s:=gross(ltrim(copy(s,9,255)));
           IF s=gross(netzwerk) THEN
             OK:=TRUE;
         END;
      END
      ELSE
      BEGIN
        IF (GROSS(COPY(s,1,7))='#DEFINE') THEN
        BEGIN
          KS[c]^.ConnScriptData^.DIGI:=Einstieg;
          KS[c]^.ConnScriptData^.USER:=CALL;
          IF KS[c]^.ConnScriptData^.Script=NIL THEN
          BEGIN
            KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;;
            DISPOSE(KS[c]^.ConnScriptData);
            KS[c]^.ConnScriptData:=NIL;
            CLOSE(f);
            EXIT;
          END;
          mkc;
          CLOSE(f);
          EXIT;
        END ELSE
          IF S[1]='\' THEN initdest:=copy(s,2,255) ELSE
          PushQueue(KS[c]^.ConnScriptData^.Script,0,s);
      END;
    END;
    IF NOT OK THEN
    BEGIN
      KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
      DISPOSE(KS[c]^.ConnScriptData);
      KS[c]^.ConnScriptData:=NIL;
      CLOSE(f);
      EXIT;
    END;
  END ELSE
  BEGIN
    KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
    DISPOSE(KS[c]^.ConnScriptData);
    KS[c]^.ConnScriptData:=NIL;
    EXIT;
  END;
  IF NOT OK THEN
  BEGIN
    KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
    DISPOSE(KS[c]^.ConnScriptData);
    KS[c]^.ConnScriptData:=NIL;
    CLOSE(f);
    EXIT;
  END ELSE
  BEGIN
    KS[c]^.ConnScriptData^.DIGI:=Einstieg;
    KS[c]^.ConnScriptData^.USER:=CALL;
    IF KS[c]^.ConnScriptData^.Script=NIL THEN
    BEGIN
      KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
      DISPOSE(KS[c]^.ConnScriptData);
      KS[c]^.ConnScriptData:=NIL;
      CLOSE(f);
      EXIT
    END;
  END;
  CLOSE(f);
  mkc;
END;

PROCEDURE DoRouting(c:BYTE;S:STRING);
VAR B:BYTE;
BEGIN
  IF S='' THEN exit;
  S:=FilterCRLF(S);
  IF (KS[c]=NIL) OR (KS[c]^.ConnScriptData=NIL) THEN
     EXIT;
  { IF (CurrIxTime-KS[c]^.ConnScriptData^.StartTime)>Router_TimeOut THEN
  BEGIN
    IF KS[C]^.ConnScriptData^.Script<>NIL THEN
      PurgeQueue( KS[C]^.ConnScriptData^.script);
    KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
    DISPOSE(KS[c]^.ConnScriptData);
    KS[c]^.ConnScriptData:=NIL;
    KS[c]^.FLAGS:=KS[c]^.FLAGS OR KS_DISCREQ;
    EXIT;
  END;}
  FOR B:=1 TO 5 DO
  WITH KS[c]^.ConnSCriptData^ DO
  BEGIN
    IF POS(ConnMessages[b],s)>0 THEN
    BEGIN
      IF Script<>NIL THEN
        LoadNextRoute(C)
      ELSE
      BEGIN
        KS[c]^.Status:=KS[c]^.ConnScriptData^.LastStatus;
        HandleConnect(c,'CONNECTED to '+USER);
        DISPOSE(KS[c]^.ConnScriptData);
        KS[c]^.ConnScriptData:=NIL;
        EXIT;
      END;
      PutData(c,Conn_Befehl+#13);
      EXIT;
    END;
    IF POS(DiscMessages[b],S)>0 THEN
    BEGIN
      IF Script<>NIL THEN
        PurgeQueue(script);
      KS[c]^.STatus:=KS[c]^.ConnScriptData^.LastStatus;
      DISPOSE(KS[c]^.ConnScriptData);
      KS[c]^.ConnScriptData:=NIL;
      KS[c]^.FLAGS:=KS[c]^.FLAGS OR KS_DISCREQ;
      EXIT;
    END;
  END;
END;


END.