(***************************************************************************)

(**                             UNIT VJ_TFPCX                             **)

(***************************************************************************)

(* Unit zur TFPCX-Ansteuerung per Software-Interrupt.                      *)

(*                                                                         *)

(*   (C) 1995 Jochen Sarrazin, DG6VJ                                       *)

(*   Weitergabe bedarf der ausdrcklichen Genemigung des Autors            *)

(*                                                                         *)

(* Diese Unit ist Teil des MCUT-Sourcecodes, alle Rechte liegen bei        *)

(* DG4IAD und DG6VJ                                                        *)

(*                                                                         *)

(*  dg6vj, jan 96                                                          *)

(***************************************************************************)

UNIT VJ_TFPCX;





INTERFACE



(*

 * TFPCX_Installed

 * Testet, ob ein TFPCX installiert ist und merkt sich den Softwareinterrupt

 * zur Kommunikation.

 *)

FUNCTION TFPCX_Installed: BOOLEAN;



(*

 * TFPCX_DataAvailable

 * Testet ob der TFPCX-Treiber Daten frs Anwenderprogramm hat.

 *)

FUNCTION TFPCX_DataAvailable: BOOLEAN;



(*

 * TFPCX_GetChar

 * Liest ein Zeichen aus dem TFPCX-Puffer ein. NUR aufrufen, wenn

 * TFPCX_DataAvailable TRUE geliefert hat!

 *)

FUNCTION TFPCX_GetChar: CHAR;



(*

 * TFPCX_SendChar

 * Ein Zeichen an den TFPCX senden

 *)

PROCEDURE TFPCX_SendChar(Zeichen: CHAR);





IMPLEMENTATION



USES DOS;



{Globale Variablen in der Unit}

VAR TFPCX_INT: BYTE;    {Interrupt zur TFPCX-Kommunikation}





(*

 * TFPCX_Installed

 * Testet, ob ein TFPCX installiert ist und merkt sich den Softwareinterrupt

 * zur Kommunikation.

 *)

FUNCTION TFPCX_Installed: BOOLEAN;

VAR Int: WORD;

    TstStrg: STRING[4];

    Idx: BYTE;          {Feld-Index}

    Offs,Segm: WORD;    {Offset- und Segment-Adresse des Interrupt-Vectors}

BEGIN

  {Das TFPCX benutzt einen Interrupt zwischen $40 und $FF zur Kommunikation

   mit dem Anwenderprogramm (Standard ist $FD). Bei einem Offset von 2

   Bytes nach der TFPCX-Interrupt-Routine steht der String 'N5NX' (An 3.

   Stelle). So kann erkannt werden, auf welchem Software-Interrupt der

   TFPCX installiert ist.



   Hintergrund:

   Jeder Interrupt-Vector wird im Segment 0h ab dem Offset 0h abgespeichert.

   Er wird als FAR-Zeiger angegeben (2 Words=4 Bytes). Deshalb auch

   IntOfs := Int*4;

   In dieser Tabelle stehen die Zeiger auf die eigentliche Interrupt-Routine.}



  {Variablen-Initialisierung}

  TFPCX_INT := 0;



  {Es sind Interrupts zwischen $40 und $FF mglich}

  Int := $40;

  WHILE (Int <= $FF) AND (TFPCX_INT = 0) DO BEGIN

    TstStrg[0] := #4;   {Lnge des Teststrings ist immer 4 Bytes}



    Offs := MemW[0:Int*4];

    Segm := MemW[0:Int*4+2];

    FOR Idx := 1 TO 4 DO TstStrg[Idx] := UpCase(Chr(Mem[Segm:Offs+Idx+2]));

    IF TstStrg = 'N5NX' THEN TFPCX_INT := Int;

    inc(Int);

  END;

  TFPCX_Installed := TFPCX_INT <> 0;

END;



(*

 * TFPCX_DataAvailable

 * Testet ob der TFPCX-Treiber Daten frs Anwenderprogramm hat.

 *)

FUNCTION TFPCX_DataAvailable: BOOLEAN;

VAR Regs: Registers;

BEGIN

  Regs.AH := 1;         {Test, ob Zeichen vorhanden sind}

  Intr(TFPCX_Int,Regs); {Interruptaufruf}

  TFPCX_DataAvailable := Regs.AX = 1;

END;



(*

 * TFPCX_GetChar

 * Liest ein Zeichen aus dem TFPCX-Puffer ein. NUR aufrufen, wenn

 * TFPCX_DataAvailable TRUE geliefert hat!

 *)

FUNCTION TFPCX_GetChar: CHAR;

VAR Regs: Registers;

BEGIN

  Regs.AH := 2;         {Zeichen vom TFPCX holen}

  Intr(TFPCX_Int,Regs);

  TFPCX_GetChar := Char(Regs.AL);

END;



(*

 * TFPCX_SendChar

 * Ein Zeichen an den TFPCX senden

 *)

PROCEDURE TFPCX_SendChar(Zeichen: CHAR);

VAR Regs: Registers;

BEGIN

  Regs.AH := 3;

  Regs.AL := Byte(Zeichen);

  Intr(TFPCX_Int,Regs);

END;





END.