(***************************************************************************)

(*                               UNIT VJ_V24                               *)

(***************************************************************************)

(* Unit fr serielle Kommunikation.                                        *)

(*                                                                         *)

(*   (C) 1995 Jochen Sarrazin, DG6VJ                                       *)

(*   Weitergabe bedarf der ausdrcklichen Genemigung des Autors            *)

(*                                                                         *)

(* Diese Unit ist Teil des MCUT-Sourcecodes, alle Rechte liegen bei        *)

(* DG4IAD und DG6VJ                                                        *)

(*                                                                         *)

(* History:                                                                *)

(*   22.01.96  1.1  FIFO-Ansteuerung                                       *)

(*                                                                         *)

(*  dg6vj, jan 96                                                          *)

(***************************************************************************)

UNIT VJ_V24;



INTERFACE



CONST VJ_V24_Version = '1.1';



      {Standard-Adressen und Interrupts der Schnistellen}

      COMAdr: ARRAY[1..4] OF WORD = ($3F8,$2F8,$3E8,$2E8);

      COMIrq: ARRAY[1..4] OF BYTE = (04,03,04,03);





(*

 * V24_Init

 * Initialisiert die Schnittstelle.

 *

 * Das aufrufende Programm mu selbst aufpassen, da ein COM-Port nicht fter

 * initialisiert wird.

 *

 * Rckgabe: Bitfeld

 *   Bit 0 = FIFO gefunden und aktiviert

 *)

FUNCTION V24_Init(Baud: LONGINT; Datenbits: BYTE; Parity: CHAR; Stopbits: BYTE; Comport: BYTE): WORD;





(*

 * V24_Deinit

 * Stellt wieder die alte Interrupt-Adresse ein.

 *)

PROCEDURE V24_Deinit;





(* V24_DataAvailable

 * Kontrolle, ob ein Byte zum Abholen vorhanden ist. Rckgabe: TRUE, wenn

 * eins da ist, FALSE wenn keins da ist.

 *)

FUNCTION V24_DataAvailable: BOOLEAN;





(* V24_GetChar

 * Liest ein Zeichen von der Schnittstelle. Darf nur aufgerufen werden,

 * wenn V24_DataAvailable TRUE geliefert hat!

 *)

FUNCTION V24_GetChar: CHAR;





(*

 * V24SendChar

 * Senden eines Zeichens. Es wird vorher geguckt, ob ein Zeichen zum

 * abholen bereit ist. Wenn ja wird es in den RX-Puffer geschrieben.

 *)

PROCEDURE V24_SendChar(Zeichen: CHAR);





(*

 * V24_PortAvailable

 * Testet ob eine Schnitstelle an der angegebenen Adresse vorhanden ist

 *)

FUNCTION V24_PortAvailable(Adresse: WORD): BOOLEAN;





(*

 * V24_PufferInhalt

 * Wieviele Bytes (Zeichen) sind im Puffer?

 *)

FUNCTION V24_PufferInhalt: WORD;







IMPLEMENTATION



USES DOS;



(************************** Lokale Variablen der Unit **********************)

CONST PuffSize = 4096;            {Gre des Ringpuffers}





VAR V24_Adr: WORD;

    V24_Irq: BYTE;

    OrigIntVector: Pointer;     {Original-Interruptvektor}

    FIFO: BYTE;

    PufferInhalt: WORD;

    Ueberlauf: BOOLEAN;

    Puffer: ARRAY[1..PuffSize+1] OF BYTE;        {Ringpuffer frs RX}

    AktPuffWrite: WORD;         {Aktuelle Stelle im Puffer (schreiben)}

    AktPuffRead: WORD;          {Aktuelle Stelle im Puffer (lesen)}

    {Wenn AktPuffWrite=AktPuffRead ist, hat entweder ein Overflow

     stattgefunden, oder alle Bytes wurden aus dem Puffer gelesen.

     Abhngig von der Prozedur, wo es passiert ist. (V24_GetChar oder

     V24_WriteByteBufferInt)}





Const

{- IRQ-Controller Portadressen --------------------------------------}

 MASTER_PIC      =    $20;             { Basisadresse des Master-PIC }

 SLAVE_PIC       =    $A0;              { Basisadresse des Slave-PIC }

 IRQ_MASK        =    $01;               { Offset zu Maskierungsport }



{- IRQ-Kommandos ----------------------------------------------------}

 EOI             =    $20;       { Unspezifizierter End of Interrupt }



 MASTER_FIRST_VECTOR = $08;        { Software-Vektoren der Hardware- }

 SLAVE_FIRST_VECTOR  = $70;        { Interrupts                      }





{********************************************************************}

{ irq_Enable : Hardware-Interrupt zulassen                           }

{*------------------------------------------------------------------*}

{ Eingabe : iIRQ - Nummer des Hardware-Interrupts (0-15), der zu-    }

{                  gelassen werden soll                              }

{*------------------------------------------------------------------*}

{ Info : Das An- und Abschalten von IRQ's geschieht durch            }

{        bermittlung eines OCW1 (Operation Control Word) genannten  }

{        Bytes an den Basisport des IRQ-Controllers.                 }

{********************************************************************}

Procedure irq_Enable( iIRQ : Integer );



var iPort : Integer;



Begin

  {-- zunchst Port Adresse des zustndigen PIC's ermitteln ---------}

  { ( 0-7 = MASTER_PIC , 8-15 = SLAVE_PIC )                          }

  if iIRQ <= 7 then iPort := MASTER_PIC

               else iPort := SLAVE_PIC ;

  iPort := iPort + IRQ_MASK;            { Maskierungs-Port auswhlen }



  iIRQ := iIRQ and $0007;      { PIC-Interruptnummer ermitteln (0-7) }

                               { Bit Lschen -> Interrupt zugelassen }

  port[iPort] := port[iPort] and not ( 1 shl iIRQ );

End;



{********************************************************************}

{ irq_Disable : Hardware-Interrupt unterdrcken                      }

{*------------------------------------------------------------------*}

{ Eingabe : iIRQ - Nummer des Hardware-Interrupts (0-15) der ver-    }

{                  boten werden soll                                 }

{********************************************************************}

Procedure irq_Disable( iIRQ : Integer );



var iPort : Integer;

Begin                 

  {-- zunchst Port Adresse des zustndigen PIC's ermitteln ---------}

  { ( 0-7 = MASTER_PIC , 8-15 = SLAVE_PIC )                          }

  if iIRQ <= 7 then iPort := MASTER_PIC

               else iPort := SLAVE_PIC ;



  iPort := iPort + IRQ_MASK;            { Maskierungs-Port auswhlen }



  iIRQ := iIRQ and $0007;      { PIC-Interruptnummer ermitteln (0-7) }

                                  { Bit Setzen -> Interrupt gesperrt }

  port[iPort] := port[iPort] or ( 1 shl iIRQ );

End;







(*

 * V24_WriteByteBuffer_Int

 * Dies ist eine Interrupt-Prozedur, die jedesmal aufgerufen wird, wenn

 * ein Byte empfangen wird. Sie schreibt es in den Ringpuffer.

 *)



{$F+} PROCEDURE V24_WriteByteBuffer_Int; {$F-} Interrupt; Assembler;

ASM

  CLI                   {Keine Interrupt-Aufrufe whrend der Interrupt-Proc}





  {******************* Test, ob Byte vorhanden ist ************************}

  @TestByteVorh:

  MOV DX,V24_Adr

  ADD DX,5              {Register 5: Leitungstatusregister}

  IN AL,DX

  AND AL,1              {Bit 0 maskieren}

  CMP AL,1              {Gucken, ob Bit 0 gesetzt war}

  JNE @NixDa            {Wenn nicht liegt nix an}



  {************************** Byte auslesen *******************************}

  MOV DX,V24_Adr        {Register 0: Datenein/ausgaberegister}

  IN AL,DX              {Ein Byte einlesen}



  {******** Adresse des Ringbuffer errechnen, wo das Byte hin soll ********}

  LEA SI,Puffer         {Adresse des Puffers nach ES:SI}

  ADD SI,AktPuffWrite   {Write-Offset dazu addieren}



  {******************** Byte in den Puffer schreiben **********************}

  MOV DS:[SI],AL



  {************************ PufferInhalt hochzhlen ***********************}

  INC PufferInhalt



  {*********************** Write-Offset hochzhlen ************************}

  CMP AktPuffWrite,PuffSize     {Test, ob ein Umbruch im Puffer stattfindet}

  JGE @Umbruch

  INC AktPuffWrite

  JMP @Weiter

  @Umbruch:

  MOV AktPuffWrite,1

  @Weiter:



  {*************************** Test auf berlauf **************************}

  CMP PufferInhalt,PuffSize

  JG @Ueberlauf

  JMP @Ende

  @Ueberlauf:

  {berlauf - Flag setzen und LeseOffset hochzhlen, damit die letzten

   PuffSize Zeichen im Puffer sind}

  DEC PufferInhalt      {Wegen berlauf wurde ein Zeichen im Puffer gelscht}

  MOV PufferInhalt,PuffSize

  MOV Ueberlauf, 1      {berlauf-Flag auf TRUE setzen}

  CMP AktPuffRead,PuffSize {Test auf umbruch}

  JGE @AuchUmbruch

  INC AktPuffRead       {Keinen Umbruch}

  JMP @Ende             {Fertig}

  @AuchUmbruch:         {Umbruch!}

  MOV AktPuffRead,1     {Wieder am Anfang des Puffers weiter machen}



  @Ende:



  {Wenn ein FIFO-Baustein gefunden wurde, wird geguckt, ob noch ein Byte

   im Puffer wartet}

  CMP FIFO,0

  JE @NixDa

  JMP @TestByteVorh



  @NixDa:               {Keine Daten warten}





  {Dem Interruptcontroller bescheid stoen, da der Interrupt abgearbeitet

   wurde.}

  MOV AL,V24_Irq

  CMP AL,8              {Interrupt grer als 7 benutzt?}

  JB @NoSecond          {Nein -> dem zweiten Controller nix sagen}

  MOV DX,$A0            {Port-Adresse des Slave-Irq-Controllers}

  MOV AL,$20            {Unspezifizierter End of Interrupt}

  OUT DX,AL



  @NoSecond:

  MOV DX,$20            {Port-Adresse vom Slave-Irq-Controller}

  MOV AL,$20            {Unspezifizierter end of Interrupt}

  OUT DX,AL



  STI                   {Interrupt-Aufrufe sind wieder erlaubt}

END;





(*

 * V24_Init

 * Initialisiert die Schnittstelle.

 *)

FUNCTION V24_Init(Baud: LONGINT; Datenbits: BYTE; Parity: CHAR; Stopbits: BYTE; Comport: BYTE): WORD;

VAR CalcBaud: WORD;

    Data: BYTE;

    iVect: INTEGER;

    lpOldHandler: Pointer;

    f: Text;

    Rueck: WORD;                {Funktionsrckgabe}

BEGIN

  Rueck := 0;

  FIFO := 0;



  {************** Adresse und Interrupt global sichern ********************}

  V24_Adr := COMAdr[Comport];

  V24_Irq := COMIrq[Comport];



  {******* in Register 3 (Datenformatregister) Bit 7 (DLAB) setzen ********}

  Port[V24_Adr+3] := Port[V24_Adr+3] OR 128;



  {************************** Baudrate setzen *****************************}

  CalcBaud := Round(115200/Baud);

  Port[V24_Adr+0] := Lo(CalcBaud);

  Port[V24_Adr+1] := Hi(CalcBaud);



  {********* DLAB wieder auf 0, sowie Daten- und Steuerbits setzen ********}

  Data := Datenbits - 5;        {Bit 0-1: Wortlnge

                                          00 = 5, 01 = 6, 10 = 7, 11 = 8 Bit}



  IF Stopbits=2 THEN Data := Data OR 4; {Bit 2: Stopbits. 0=1 Bit, 1=2 Bit}



  Parity := UpCase(Parity);  {Paritts-Prfung}

  CASE Parity OF

    'G': Data := Data OR 24; {Gerade Paritt. Bit 3 und 4 gesetzt}

    'U': Data := Data OR 8;  {Ungerade Paritt. Bit 3 gesetzt, 4 nicht}

  END;



  Port[V24_Adr+3] := Data;





  {********************* Interrupt-Vektor neu setzen *******************}

  IF V24_Irq <= 7 THEN iVect := (MASTER_FIRST_VECTOR + V24_Irq)

                  ELSE iVect := (SLAVE_FIRST_VECTOR + (V24_Irq and $7));

  irq_Disable(V24_Irq);

  ASM CLI END;

  GetIntVec(iVect,OrigIntVector);               {Alten Vector retten}

  SetIntVec(iVect,@V24_WriteByteBuffer_Int);    {Neuen Vector setzen}

  ASM STI END;

  irq_Enable(V24_Irq);



  {******************** Interrupt bei RX freigeben *********************}

  Port[V24_Adr+1] := 1;



  {********** Test, ob FIFO vorhanden. Wenn ja -> einschalten **********}

  Port[V24_Adr+2] := 1+192;     {192 = Bit 6,7 gesetzt: Int nach 14

                                 empfangenen Zeichen, 1 = Bit 0: FIFO an}

  IF Port[V24_Adr+2] >=64 THEN BEGIN    {FIFO vorhanden}

    FIFO := 1;

    Rueck := Rueck OR 1;        {Bit 0 setzen}

  END;



  {*************** UART scharf schalten (Modemregister) ****************}

  Port[V24_Adr+4] := 8;



  {************************** Funktions-Rckgabewert *******************}

  V24_Init := Rueck;

END;





(*

 * V24_Deinit

 * Stellt wieder die alte Interrupt-Adresse ein.

 *)

PROCEDURE V24_Deinit;

VAR iVect: INTEGER;

BEGIN

  IF V24_Irq <= 7 THEN iVect := (MASTER_FIRST_VECTOR + V24_Irq)

                  ELSE iVect := (SLAVE_FIRST_VECTOR + (V24_Irq and $7));

  irq_Disable(V24_Irq);

  ASM CLI END;

  SetIntVec(iVect,OrigIntVector);       {Neuen Vector setzen}

  ASM STI END;

  irq_Enable(V24_Irq);



  {******************* Es werden keine Interrupts ***********************}

  {***************** von der Schnittstelle ausgelst ********************}

  Port[V24_Adr+1] := 0;



  Port[V24_Adr+4] := Port[V24_Adr+4] AND 8;



  {************************* FIFO ausschalten ***************************}

  Port[V24_Adr+2] := 0;

END;





(*

 * V24_DataAvailable

 * Kontrolle, ob ein Byte zum Abholen vorhanden ist. Rckgabe: TRUE, wenn

 * eins da ist, FALSE wenn keins da ist.

 *)

FUNCTION V24_DataAvailable: BOOLEAN; Assembler;

ASM

  MOV DX, PufferInhalt

  CMP DX,0

  JE @Leer



  MOV AL,1

  JMP @Feddich

  @Leer:

  MOV AL,0



  @Feddich:

END;





(* V24_GetChar

 * Liest ein Zeichen von der Schnittstelle. Darf nur aufgerufen werden,

 * wenn V24_DataAvailable TRUE geliefert hat!

 *)

FUNCTION V24_GetChar: CHAR; Assembler;

ASM

  LEA SI,Puffer         {Adresse des Puffers nach SI}

  ADD SI,AktPuffRead    {Offset (Ringpuffer) dazu}

  MOV AL,DS:[SI]        {Zeichen auslesen}



  {********************* Lese-Offset eins hher **************************}

  INC AktPuffRead

  CMP AktPuffRead,PuffSize {Kontrolle, ob ein berlauf von PuffSize nach 1

                         stattgefunden hat}

  JBE @Weiter

  MOV AktPuffRead,1     {berlauf -> bei 1 weitermachen}



  @Weiter:              {Ne, kein berlauf}

  DEC PufferInhalt      {Ein Byte weniger ist jetzt noch im Puffer}

END;





(*

 * V24SendChar

 * Senden eines Zeichens. Es wird vorher geguckt, ob ein Zeichen zum

 * abholen bereit ist. Wenn ja wird es in den RX-Puffer geschrieben.

 *)

PROCEDURE V24_SendChar(Zeichen: CHAR); Assembler;

ASM



  {******************* Test, ob Byte vorhanden ist ************************}

  @TestByteVorh:

  MOV DX,V24_Adr

  ADD DX,5              {Register 5: Leitungstatusregister}

  IN AL,DX

  AND AL,1              {Bit 0 maskieren}

  CMP AL,1              {Gucken, ob Bit 0 gesetzt war}

  JNE @NixDa            {Wenn nicht liegt nix an}



  {************************** Byte auslesen *******************************}

  MOV DX,V24_Adr        {Register 0: Datenein/ausgaberegister}

  IN AL,DX              {Ein Byte einlesen}



  {******** Adresse des Ringbuffer errechnen, wo das Byte hin soll ********}

  LEA SI,Puffer         {Adresse des Puffers nach ES:SI}

  ADD SI,AktPuffWrite   {Write-Offset dazu addieren}



  {******************** Byte in den Puffer schreiben **********************}

  MOV DS:[SI],AL



  {************************ PufferInhalt hochzhlen ***********************}

  INC PufferInhalt



  {*********************** Write-Offset hochzhlen ************************}

  CMP AktPuffWrite,PuffSize     {Test, ob ein Umbruch im Puffer stattfindet}

  JGE @Umbruch

  INC AktPuffWrite

  JMP @Weiter

  @Umbruch:

  MOV AktPuffWrite,1

  @Weiter:



  {*************************** Test auf berlauf **************************}

  CMP PufferInhalt,PuffSize

  JG @Ueberlauf

  JMP @Ende

  @Ueberlauf:

  {berlauf - Flag setzen und LeseOffset hochzhlen, damit die letzten

   PuffSize Zeichen im Puffer sind}

  DEC PufferInhalt      {Wegen berlauf wurde ein Zeichen im Puffer gelscht}

  MOV PufferInhalt,PuffSize

  MOV Ueberlauf, 1      {berlauf-Flag auf TRUE setzen}

  CMP AktPuffRead,PuffSize {Test auf umbruch}

  JGE @AuchUmbruch

  INC AktPuffRead       {Keinen Umbruch}

  JMP @Ende             {Fertig}

  @AuchUmbruch:         {Umbruch!}

  MOV AktPuffRead,1     {Wieder am Anfang des Puffers weiter machen}



  @Ende:



  {Wenn ein FIFO-Baustein gefunden wurde, wird geguckt, ob noch ein Byte

   im Puffer wartet}

  CMP FIFO,0

  JE @NixDa

  JMP @TestByteVorh





  @NixDa:               {Keine Daten warten}



  {**************************** Byte senden *******************************}

  MOV DX,V24_Adr        {Register 0: Sendehalteregister}

  MOV AL,Zeichen        {Zu sendendes Zeichen nach AL}

  OUT DX,AL



  {********************* Warten, bis UART besttigt ***********************}

  @Schleife:

  MOV DX,V24_Adr

  ADD DX,5              {Register 5: Leitungsstatusregister}

  IN AL,DX              {Leitungsstatus einlesen}

  AND AL,64             {Bit 6 maskieren: Senderegister und Sendehalte-

                         register sind leer}

  CMP AL,64

  JNE @Schleife          {Wenn das Bit nicht gesetzt ist -> springe zu

                         @Schleife und warte, bis es gesetzt ist}

END;





(*

 * V24_PortAvailable

 * Testet ob eine Schnitstelle an der angegebenen Adresse vorhanden ist

 *)

FUNCTION V24_PortAvailable(Adresse: WORD): BOOLEAN;

VAR LSR: BYTE;

BEGIN

  LSR := Port[Adresse+5];    {LSR = Line Status Register}

  {Suchbedingung: LSR mu Bitmuster $60 enthalten}

  IF ((LSR AND $60)=$60) AND (LSR<>$FF) THEN V24_PortAvailable := TRUE

                                        ELSE V24_PortAvailable := FALSE;

END;





(*

 * V24_PufferInhalt

 * Wieviele Bytes (Zeichen) sind im Puffer?

 *)

FUNCTION V24_PufferInhalt: WORD;

BEGIN

  V24_PufferInhalt := PufferInhalt;

END;





BEGIN

  {Variablen Initialisieren}

  AktPuffWrite := 0;

  AktPuffRead := 0;

  PufferInhalt := 0;

  Ueberlauf := FALSE;

END.

